use CorpDBExpanded;
go
set statistics io on;

----------------------------------------------------------------------------------------------------
-- Get an ACTUAL execution plan on this statement.
----------------------------------------------------------------------------------------------------
select distinct top (10) c.FirstName
from dbo.Customer c
where c.LastName = 'Ming'
order by c.FirstName asc
option (recompile);
-- Observe that estimated vs actual number of row is significantly different.
-- The nonclustered index on LastName is used, with a key lookup.
-- A nested loops join is chosen. The key lookup happens 930K times.
-- Estimated query cost is very small (0.12).
-- Required 2,861,099 reads from Customer table (at 8K each = 21.8 GB).
-- The sort operation spilled to tempdb.

----------------------------------------------------------------------------------------------------
-- Now update stats on the LastName index.
----------------------------------------------------------------------------------------------------
update statistics dbo.Customer (idx_Customer__LastName) with fullscan;

----------------------------------------------------------------------------------------------------
-- Re-execute query and observe the ACTUAL execution plan on this statement.
----------------------------------------------------------------------------------------------------
select distinct top (10) c.FirstName
from dbo.Customer c
where c.LastName = 'Ming'
order by c.FirstName asc
option (recompile);
-- Now actual vs. estimated rows match.
-- Does a clustered index scan.
-- Query cost is now 15.4.
-- Only does 10,391 reads.
-- BUT ... despite a higher apparent query cost, this is not correct due to bad CE.

----------------------------------------------------------------------------------------------------
-- To get a better estimate of the nested loops / key lookup query actually cost, we
-- can get ESTIMATED execution plan on this statement (now that stats have been updated).
----------------------------------------------------------------------------------------------------
select distinct top (10) c.FirstName
from dbo.Customer c with (index = idx_Customer__LastName)
where c.LastName = 'Ming'
order by c.FirstName asc
option (recompile);
-- Query cost = 192
